/**
 * Fisher-Yates shuffle.
 * Defaults to in-place sorting, but can sort into a new one via {@code into}.
 * @see https://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle
 */
export const shuffle = (list, { into = list, random01 = Math.random } = {}) => {
    const n = list.length;
    for (let i = 0; i < n; i++) {
        const j = Math.trunc(random01() * (i + 1));
        const t = list[i];
        into[i] = into[j];
        into[j] = t;
    }
    return into;
};
export const toShuffled = (list, config) => shuffle(list, { ...config, into: Array(list.length) });
//# sourceMappingURL=shuffle.js.map