import type { Resolve } from "@rickosborne/typical/ts/index.js";
export type ReadOnlyPropertyDescriptor = {
    writable: false;
};
export type PropertyDescriptorFlags = {
    configurable?: boolean;
    enumerable?: boolean;
    writable?: boolean;
};
export type ValuePropertyDescriptor<T> = PropertyDescriptorFlags & {
    value: T;
};
export type GetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get(): T;
    set?: undefined;
};
export type SetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get?: undefined;
    set(t: T): void;
};
export type GetSetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get(): T;
    set(t: T): void;
};
export type TypedPropertyDescriptor<T> = ValuePropertyDescriptor<T> | GetPropertyDescriptor<T> | SetPropertyDescriptor<T> | GetSetPropertyDescriptor<T>;
type IfReadOnly<T, D extends TypedPropertyDescriptor<unknown>> = D extends ReadOnlyPropertyDescriptor | GetPropertyDescriptor<unknown> ? T : never;
type IfReadWrite<T, D extends TypedPropertyDescriptor<unknown>> = D extends ReadOnlyPropertyDescriptor | GetPropertyDescriptor<unknown> ? never : T;
type RenameIfWriteOnly<T extends string, D extends TypedPropertyDescriptor<unknown>> = D extends SetPropertyDescriptor<unknown> ? `${T}IsWriteOnly` : never;
type ReadOnlyKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in K]: IfReadOnly<key, R[key]>;
}[K] : never;
type ReadWriteKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in K]: IfReadWrite<key, R[key]>;
}[K] : never;
type WriteOnlyKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in string & K]: RenameIfWriteOnly<key, R[key]>;
}[string & K] : never;
export type PropertyFromDescriptor<K extends string, D extends TypedPropertyDescriptor<unknown>> = D extends TypedPropertyDescriptor<infer V> ? ({
    readonly [key in IfReadOnly<K, D>]: V;
} & {
    [key in IfReadWrite<K, D>]: V;
} & {
    [key in `${RenameIfWriteOnly<K, D>}`]?: undefined;
}) : never;
export type PropertiesFromDescriptors<R extends Record<string, TypedPropertyDescriptor<unknown>>> = {
    readonly [K in ReadOnlyKeys<R>]: R[K] extends TypedPropertyDescriptor<infer V> ? V : never;
} & {
    [K in ReadWriteKeys<R>]: R[K] extends TypedPropertyDescriptor<infer V> ? V : never;
} & {
    [K in WriteOnlyKeys<R>]?: undefined;
};
/**
 * A more type-friendly version of {@link Object.defineProperty}.
 */
export declare const addProperty: <T, K extends string, V, D extends TypedPropertyDescriptor<V>>(target: T, key: K, descriptor: D) => Resolve<T & PropertyFromDescriptor<K, D>>;
/**
 * A more type-friendly version of {@link Object.defineProperties}.
 */
export declare const addProperties: <T, K extends string, R extends Record<K, TypedPropertyDescriptor<unknown>>>(target: T, descriptors: R) => Resolve<T & PropertiesFromDescriptors<R>>;
export {};
