export type ShuffleConfig<T> = {
    /**
     * Target array.  Defaults to the given array, shuffling in place.
     */
    into?: T[] | undefined;
    /**
     * Generate a random float in the range {@code [0..1)}.
     */
    random01?: () => number;
};
/**
 * Fisher-Yates shuffle.
 * Defaults to in-place sorting, but can sort into a new one via {@code into}.
 * @see https://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle
 */
export declare const shuffle: <T>(list: T[], { into, random01 }?: ShuffleConfig<T>) => T[];
export declare const toShuffled: <T>(list: T[], config?: Omit<ShuffleConfig<T>, "into"> | undefined) => T[];
